"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.terraformAwsCli = exports.AWSCloudProvider = void 0;
const infra_aws_1 = require("@goldstack/infra-aws");
const utils_terraform_1 = require("@goldstack/utils-terraform");
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const tfState_1 = require("./tfState");
const crypto_1 = __importDefault(require("crypto"));
const getRemoteStateConfig = (config, userName) => {
    const userConfig = config.remoteState.filter((u) => u.user === userName);
    if (userConfig.length === 0) {
        throw new Error(`Cannot find aws user ${userName} in project terraform config`);
    }
    return userConfig[0];
};
class AWSCloudProvider {
    constructor(credentials, awsConfig) {
        this.generateEnvVariableString = () => {
            return (`-e AWS_ACCESS_KEY_ID=${this.user.accessKeyId} ` +
                `-e AWS_SECRET_ACCESS_KEY=${this.user.secretAccessKey} ` +
                `-e AWS_SESSION_TOKEN=${this.user.sessionToken || ''} ` +
                `-e AWS_DEFAULT_REGION=${aws_sdk_1.default.config.region} `);
        };
        this.setEnvVariables = () => {
            process.env.AWS_ACCESS_KEY_ID = this.user.accessKeyId;
            process.env.AWS_SECRET_ACCESS_KEY = this.user.secretAccessKey;
            process.env.AWS_DEFAULT_REGION = aws_sdk_1.default.config.region;
            process.env.AWS_SESSION_TOKEN = this.user.sessionToken || '';
        };
        this.getTfStateVariables = (deployment) => {
            const remoteStateConfig = getRemoteStateConfig(this.remoteStateConfig, deployment.awsUser);
            const bucket = remoteStateConfig.terraformStateBucket;
            if (!bucket) {
                throw new Error(`TF state bucket not defined for user ${deployment.awsUser}`);
            }
            const ddTable = remoteStateConfig.terraformStateDynamoDBTable;
            if (!ddTable) {
                throw new Error(`TF state DynamoDB table not defined for user ${deployment.awsUser}`);
            }
            if (!aws_sdk_1.default.config.region) {
                throw new Error('AWS region not defined');
            }
            const tfKey = deployment.tfStateKey;
            if (!tfKey) {
                throw new Error('Terraform state key not defined');
            }
            return [
                ['bucket', bucket],
                ['key', `${tfKey}`],
                ['region', aws_sdk_1.default.config.region],
                ['dynamodb_table', ddTable],
            ];
        };
        this.user = credentials;
        this.remoteStateConfig = awsConfig;
    }
}
exports.AWSCloudProvider = AWSCloudProvider;
const terraformAwsCli = async (args) => {
    const deploymentName = args[1];
    const deployment = (0, infra_aws_1.readDeploymentFromPackageConfig)(deploymentName);
    const credentials = await (0, infra_aws_1.getAWSUser)(deployment.awsUser);
    const awsTerraformConfig = (0, infra_aws_1.assertTerraformConfig)(deployment.awsUser);
    const projectHash = crypto_1.default.randomBytes(20).toString('hex');
    const remoteStateConfig = getRemoteStateConfig(awsTerraformConfig, deployment.awsUser);
    if (!remoteStateConfig.terraformStateBucket) {
        remoteStateConfig.terraformStateBucket = `goldstack-tfstate-${projectHash}`;
        (0, infra_aws_1.writeTerraformConfig)(awsTerraformConfig);
    }
    if (!remoteStateConfig.terraformStateDynamoDBTable) {
        remoteStateConfig.terraformStateDynamoDBTable = `goldstack-tfstate-${projectHash}-lock`;
        (0, infra_aws_1.writeTerraformConfig)(awsTerraformConfig);
    }
    await (0, tfState_1.createState)({
        bucketName: remoteStateConfig.terraformStateBucket,
        dynamoDBTableName: remoteStateConfig.terraformStateDynamoDBTable,
        credentials,
    });
    (0, utils_terraform_1.terraformCli)(args, {
        provider: new AWSCloudProvider(credentials, awsTerraformConfig),
    });
};
exports.terraformAwsCli = terraformAwsCli;
//# sourceMappingURL=utilsTerraformAws.js.map